# coding = utf-8
import json
import time
from time import time as timestamp
from bs4 import BeautifulSoup
import requests
import secmail

from .helpers import constants, failed, log, print_banner, save
from . import entities, exceptions
from .device import get_device_id
from .headers import get_api_headers
from .web_socket import Callbacks, SocketHandler
from colored import attr, back, fore, style

MB = fore.MAGENTA_3A + back.BLACK      + style.BLINK
OB = fore.ORANGE_1   + back.BLACK      + style.BLINK
BO = fore.BLACK      + back.ORANGE_1   + style.BLINK
OM = fore.ORANGE_1   + back.MAGENTA_3A + style.BLINK
WB = fore.BLACK      + back.WHITE      + style.BLINK
reset = attr('reset')

debug = True

print_banner()
input ("Press enter to start >> ")

class Client(Callbacks, SocketHandler):
    def __init__(self, deviceId: str = None, proxies: dict = None, certificatePath=None, socket_trace=False, socketDebugging=False):
        self.api = "https://service.narvii.com/api/v1"
        self.authenticated = False
        self.configured = False

        self.device_id = deviceId if deviceId is not None else get_device_id()

        SocketHandler.__init__(self, self, socket_trace=socket_trace, debug=socketDebugging)
        Callbacks.__init__(self, self)
        self.proxies = proxies
        self.certificatePath = certificatePath

        self.json = None
        self.sid = None
        self.userId = None
        self.account: entities.UserProfile = entities.UserProfile(None)
        self.profile: entities.UserProfile = entities.UserProfile(None)

    def _get_json_object(self, url, key: str = None):
        response = requests.get(url, headers=self.parse_headers(), proxies=self.proxies, verify=self.certificatePath)
        json_object = json.loads(response.text)
        if response.status_code != 200:
            exceptions.CheckException(json_object)
            return None

        return json_object[key] if key is not None else json_object

    def _get_entity(self, url, cls, key: str = None):
        json_object = self._get_json_object(url, key=key)
        return json_object and cls(json_object).Self

    def _post_get_json(self, url, data=None, headers=None, success_code=200, json_data=None):
        if data is not None:
            data = json.dumps(data)
        elif json_data is not None:
            data = json_data

        if headers is None:
            headers = self.parse_headers(data=data)

        response = requests.post(url, data=data, headers=headers, proxies=self.proxies, verify=self.certificatePath)
        json_object = json.loads(response.text)
        if response.status_code != success_code:
            exceptions.CheckException(json_object)
            return None

        return json_object

    def _post(self, url, data=None, headers=None, success_code=200):
        return None if self._post_get_json(url, data, headers, success_code) is None else success_code

    def _delete(self, url):
        response = requests.delete(url, headers=self.parse_headers(), proxies=self.proxies, verify=self.certificatePath)
        if response.status_code != 200:
            exceptions.CheckException(json.loads(response.text))
            return None

        return response.status_code

    def parse_headers(self, data: str = None):
        return get_api_headers(data=data, sid=self.sid, device_id=self.device_id)

    def deviceGenerator(self) -> str:
        device = requests.get(f"http://forevercynical.com/generate/deviceid").text
        return device
        
    def verify(self, email: str, password: str, deviceId):
        url = f"{self.api}/g/s/auth/login"
        data = {
            "email": email,
            "v": 2,
            "secret": f"0 {password}",
            "deviceID": deviceId,
            "clientType": 100,
            "action": "normal",
            "timestamp": int(timestamp() * 1000)
        }
        json_object = self._post_get_json(url, data)
        self.run_socket()
        if json_object is not None:
            self.authenticated = True
            self.json = json_object
            self.sid = self.json["sid"]
            self.userId = self.json["account"]["uid"]
            self.account: entities.UserProfile = entities.UserProfile(self.json["account"]).Self
            self.profile: entities.UserProfile = entities.UserProfile(self.json["userProfile"]).Self
            self.start()
            return 200

        return None

    def request_verify_code(self, email: str):
        time.sleep(2)
        url = f"{self.api}/g/s/auth/request-security-validation"
        data = {
            "identity": email,
            "type": 1,
            "deviceID": reset_deviceId
        }
        return self._post_get_json(url, data)


    def update_password(self, email: str, new_password: str, captcha: str, link: str):
        time.sleep(2)
        url = f"{self.api}/g/s/auth/reset-password"
        data = {
            "updateSecret": f"0 {new_password}",
            "emailValidationContext": {
                "data": {
                    "code": captcha
                },
                "type": 1,
                "identity": email,
                "level": 2,
                "deviceID": reset_deviceId
            },
            "phoneNumberValidationContext": None,
            "deviceID": reset_deviceId
        }

        response = json_object = self._post(url, data)
        if response is None:
            print(f"Projected captcha: {captcha}")
            print(f"\nIf captcha matches link: \n{link}, \nand still fails.. try again later")
            failed(email)
            input("\nPress Enter to continue >>")
        if response == 200:
            save(email, new_password, reset_deviceId)
            short_device = f"{reset_deviceId[:5]}...{reset_deviceId[-5:]}"
            short_email = f"{email[:5]}...{email[-5:]}"
            short_pass = f"{new_password[:5]}...{new_password[-5:]}"
            self.print_pass(short_email, short_pass, short_device)
        return json_object

    def check_mail(self, email: str, new_password: str):
        try:
            time.sleep(.7)
            mail = secmail.SecMail()
            mails = mail.get_messages(email=email).id
            log(f"Checking inbox: {email}", do_log = debug)
            themail1 = mails[0]
            inbox = mail.read_message(email, themail1).htmlBody
            self.solve_captcha(email, new_password=new_password, inbox=inbox)
        except IndexError:
            self.check_mail(email, new_password=new_password)

    def solve_captcha(self, email: str, new_password: str, inbox):
        soup = BeautifulSoup(inbox, "lxml")
        captcha = soup.find_all("a")
        captcha = captcha[0].get("href")
        link = captcha
        captcha = constants(captcha)
        self.update_password(email=email, new_password=new_password, captcha = captcha, link = link)

    def request_pass_code(self, email: str, new_password: str, deviceId: str):
        try:
            global reset_deviceId
            reset_deviceId = deviceId
            url = f"{self.api}/g/s/auth/request-security-validation"
            data = {
                "identity": email,
                "type": 1,
                "deviceID": reset_deviceId,
                "level": 2,
                "purpose":  "reset-password"
            }
            self._post(url, data)
            log('\nPassword Request Sent..', do_log = debug)
            self.check_mail(email=email, new_password=new_password)
        except Exception as e:
            print(e)

    def print_pass(self, short_email: str, short_pass: str,  short_device: str):
        color = fore.ORANGE_1 + back.BLACK + style.BLINK
        
        print(MB + "┄"*57, (reset))
        print(color + f"\tSuccessfully Changed Password!")
        print(color + f"\tEmail: {short_email}")
        print(color + f"\tPassword: {short_pass}")
        print(color + f"\tDevice: {short_device}")
        print(MB + "┄"*57, (reset))

    def change_password(self, email: str = None, new_password: str = None, deviceId: str = None):
        time.sleep(1.5)
        self.request_pass_code(email, new_password, deviceId)

